#Tracé d'un cadran solaire de hauteur plan, à style droit fixe

import matplotlib.pyplot as plt
import numpy as np

#paramètres de départ :----------------------------------------------
latitude=np.radians(49.5)#REGLAGE LATITUDE
a=3.0#longueur du style 
largeur=18#largeur du plan, pour l'échelle horizontale
plt.rcParams["font.family"] = "Ubuntu"#ou autre police installée sur le système
plt.rcParams.update({"text.usetex": True}) #police LaTeX (optionnel)
plt.rcParams["font.size"] = 14#taille de la police

#Modèlisation déclinaison selon le rang J du jour :
def declinaison(J):
    return np.arcsin(np.sin(np.radians(23.44))*np.sin((J-81)*2*np.pi/365.2422))

#------------------------------------------------------------------------------------
#calcul de l'angle horaire maximal, d'après la latitude :
Ahmax=np.degrees(np.arccos(-np.tan(latitude)*np.tan(np.radians(23.4))))
#Ahmax=105#pour forcer Ahmax manuellement
Ah=np.radians(np.arange(0, Ahmax+15, 15))#angle horaire
J=np.arange(1, 365, 0.1)#rang du jour dans l'année
N=np.arange(0, 181, 1)#rang du jour après 21 juin
decl=declinaison(N+171)#déclinaison après le 21 juin
x=-largeur/180*N#distance horizontale au 21 juin
b=np.sqrt(a**2+x**2)#taille du style virtuel, à une date donnée

#Paramètres du graphique :
fig=plt.figure(figsize=(8, 8), tight_layout=True, edgecolor='royalblue', linewidth=2)
ax=plt.subplot(111)
plt.xlim(-largeur-0.6, 0.6)#limites de l'axe en abscisses

plt.title("Cadran de hauteur à style fixe (Latitude %s°)"%(round(np.degrees(latitude), 2)))

plt.scatter(0, 0, s=12, color='red')#point d'implantation du style
plt.text(0, 0.1, "style", ha='center', va='bottom', c='red')
plt.axvline(0, ls='--', color="purple", linewidth=0.5)
plt.axvline(-largeur, ls='--', color="purple", linewidth=0.5)

#------------------------------------------------------------------------------------------
#Calcul et tracé des lignes horaires
for H in Ah:
    h=np.arcsin(np.sin(decl)*np.sin(latitude)+np.cos(decl)*np.cos(latitude)*np.cos(H))
    y=-b*np.tan(h)
    plt.plot(x, y)
    h0hiver=np.arcsin(np.sin(min(decl))*np.sin(latitude)+np.cos(min(decl))*np.cos(latitude)*np.cos(H))
    h0ete=np.arcsin(np.sin(max(decl))*np.sin(latitude)+np.cos(max(decl))*np.cos(latitude)*np.cos(H))
    if h0ete>0:#écrire l'heure (matin)
        plt.text(0.1, -a*np.tan(h0ete), "%s"%(int(round(12-np.degrees(H)/15))), fontsize=13, va='center')
    if h0hiver >0:#écrire l'heure (soir)
        plt.text(-largeur, -largeur*np.tan(h0hiver), "%s"%(int(12+np.degrees(H)/15)), fontsize=13, va='center', ha='right')

#Demi-heures :
demies=np.radians(np.arange(7.5, 127.5, 15))
for H in demies:
    h=np.arcsin(np.sin(decl)*np.sin(latitude)+np.cos(decl)*np.cos(latitude)*np.cos(H))
    y=-b*np.tan(h)
    plt.plot(x, y, 'g--', alpha=0.5, linewidth=0.5)

#---------------------------------------------------------------
ymin, ymax=ax.get_ylim()#récupération des limites verticales (avant le tracé des graduations de hauteur)
plt.ylim(ymin, 0)

#graduations juillet à décembre :
datesS2=np.array([182, 213, 244, 274, 305, 335])#rangs des 1ers jours des mois
initialeS2=["D", "N", "O", "S", "A", "J"]#initiale du mois

gradDates=np.arange(-16, 0, 3)
ax.xaxis.set_ticks(gradDates)
ax.xaxis.set_ticklabels(initialeS2)
ax.xaxis.set_ticks(np.arange(-18, -1, 1), minor=True)#grille secondaire verticale

#graduations janvier à juin :
initialeS1=["J", "F", "M", "A", "M", "J"]#initiale du mois
datesS1=np.array([0, 30, 60, 90, 120, 150])#rangs du 1er jour du mois
xdatesS1=-(170-datesS1)/180*largeur
for n in range(len(datesS1)):
    plt.text(xdatesS1[n], ymin+0.1, initialeS1[n], ha='center')

ax.yaxis.set_ticks([])#suppression des graduations en ordonnées

plt.grid(which='major')
plt.grid(which='minor', alpha=0.5)

#Repères de saisons
plt.text(0, 0.98*ymin, "solstice d'été", rotation=90, backgroundcolor="white", ha='center')
plt.text(-largeur, 0.98*ymin, "solstice d'hiver", rotation=90, backgroundcolor="white", ha='center')

#lignes de hauteurs :
hmax=90+23.4-np.degrees(latitude)#hauteur méridienne maxi (21 juin)
gradh=np.radians(np.arange(5, hmax+5, 5))
for h in gradh:
    y=-b*np.tan(h)
    plt.plot(x, y, linewidth=0.4, color='gray', alpha=0.5)
    
#Segment de la longueur du style :
plt.plot([-largeur+1, -largeur+1+a], [ymin+1, ymin+1], lw='6', color='darkblue', solid_capstyle='butt')
plt.text(-largeur+1+a/2, ymin+0.6, 'longueur du style', ha='center')

#-------------------------------------------------------------------------------------


fig.savefig("CadranHauteurPlanStyleFixe495.pdf")
fig.savefig("CadranHauteurPlanStyleFixe495.png", dpi=300)
plt.show()
